import numpy as np


def read_fda(filename):
    with open(filename, "rb") as f:
        hdr_buf = f.read(4)
        [code] = np.frombuffer(hdr_buf, dtype=np.int32, count=1)
        if code >= 300:
            hdr_buf = f.read(12)
            [nx, ny, nz] = np.frombuffer(hdr_buf, dtype=np.int32, count=3)
        else:
            hdr_buf = f.read(8)
            [nx, ny] = np.frombuffer(hdr_buf, dtype=np.int32, count=2)
            nz = 1
        dtype = {
            300: np.uint8,
            301: np.uint32,
            302: np.float32,
            303: np.float64,
            200: np.uint8,
            201: np.uint32,
            202: np.float32,
            203: np.float64,
        }[code]
        bytes_data = f.read()
        array_data = np.frombuffer(bytes_data, dtype=dtype, count=nx * ny * nz)

    return np.reshape(array_data, (nz, nx, ny))


def write_fda(img, full_name):
    [nz, nx, ny] = img.shape

    dt = img.dtype

    if nz > 1:
        code = {np.uint8: 300, np.uint32: 301, np.float32: 302, np.float64: 303}[
            dt.type
        ]
        hb = np.array([code, nz, nx, ny])
    else:
        code = {np.uint8: 200, np.uint32: 201, np.float32: 202, np.float64: 203}[
            dt.type
        ]
        hb = np.array([code, nx, ny])
    f = open(full_name, "wb")
    f.write(hb.tobytes())
    f.write(img.tobytes())
    f.close()


def get_all_params_from_xml(xml_path):
    import xml.etree.ElementTree as ET

    root = ET.parse(xml_path).getroot()
    return {child.tag: child.attrib["Value"] for child in root}
